// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

/// {@template app_exception}
/// [AppException] is a base class for all exceptions in the wyatt architecture.
/// {@endtemplate}
abstract class AppException implements Exception {
  /// {@macro app_exception}
  const AppException([this.message]);

  final String? message;

  @override
  String toString() {
    if (message?.isNotEmpty ?? false) {
      return '$runtimeType: $message';
    } else {
      return '$runtimeType: An exception occured';
    }
  }
}

/// {@template client_exception}
/// [ClientException] is a base class for all client exceptions in the wyatt
/// architecture.
/// {@endtemplate}
class ClientException extends AppException {
  /// {@macro client_exception}
  const ClientException([super.message]);
}

/// {@template server_exception}
/// [ServerException] is a base class for all server exceptions in the wyatt
/// architecture.
/// {@endtemplate}
class ServerException extends AppException {
  /// {@macro server_exception}
  const ServerException([super.message]);
}
