// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:wyatt_continuous_deployment/src/core/task/task_cubit.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/context.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/flutter.dart';

part 'build_app_states.dart';

abstract class BuildAppTask extends TaskCubit {
  BuildAppTask({
    required this.flutter,
    required super.checkToolsUsecase,
    required super.checkConfigFileUsecase,
  });

  final Flutter flutter;

  FutureOr<void> buildApp(Context ctx);

  @override
  Future<void> onRun(Context ctx) async {
    emit(BuildAppTaskFlutterClean());

    final flutterCleanResult = await flutter.clean();
    if (flutterCleanResult.isErr) {
      return emit(TaskStateError(message: flutterCleanResult.err.toString()));
    }

    emit(BuildAppTaskFlutterPubGet());

    final flutterGetDependenciesResult = await flutter.getDependencies();
    if (flutterGetDependenciesResult.isErr) {
      return emit(
        TaskStateError(message: flutterGetDependenciesResult.err.toString()),
      );
    }

    emit(BuildAppTaskBuildApp());
    await buildApp(ctx);
  }
}
