// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:io';

import 'package:json_annotation/json_annotation.dart';

part 'ios_config.g.dart';

@JsonSerializable(
  anyMap: true,
  fieldRename: FieldRename.snake,
  checked: true,
)
class IosConfig {
  const IosConfig({
    required this.appleApiPrivateKeyFileName,
    required this.gitCertsPrivateKeyFileName,
    required this.iosFolderPath,
    required this.encryptedKeyFileName,
    required this.developerAppIdentifier,
    required this.teamName,
    required this.appleIssuerId,
    required this.gitUrl,
    required this.gitPassphrase,
    required this.developerAppId,
    required this.userName,
    required this.appleKeyId,
    required this.teamId,
    required this.provisionningProfiles,
    required this.flutterArgs,
    required this.iosKeysFileName,
  });

  factory IosConfig.fromJson(Map<dynamic, dynamic> json) =>
      _$IosConfigFromJson(json);

  Map<String, dynamic> toJson() => _$IosConfigToJson(this);

  static const defaultValues = IosConfig(
    iosFolderPath: 'ios',
    encryptedKeyFileName: 'ios_keys.zip.gpg',
    iosKeysFileName: 'ios_keys.json',
    appleApiPrivateKeyFileName: 'key.p8',
    gitCertsPrivateKeyFileName: '',
    developerAppIdentifier: 'git-key',
    teamName: '',
    appleIssuerId: '',
    gitUrl: '',
    gitPassphrase: 'match-password',
    developerAppId: '',
    userName: '',
    appleKeyId: '',
    teamId: '',
    provisionningProfiles: {},
    flutterArgs: '',
  );

  final String flutterArgs;
  final String iosFolderPath;
  final String encryptedKeyFileName;
  final String appleApiPrivateKeyFileName;
  final String gitCertsPrivateKeyFileName;

  final String iosKeysFileName;
  final String developerAppId;
  final String developerAppIdentifier;
  final String teamId;
  final String teamName;
  final String userName;

  final String appleIssuerId;
  final String appleKeyId;

  final String gitPassphrase;
  final String gitUrl;
  final Map<String, String> provisionningProfiles;

  File get encryptedKeyFile => File('$iosFolderPath/$encryptedKeyFileName');
  Directory get iosDirectory => Directory(iosFolderPath);
}
