// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_continuous_deployment/src/core/extensions/object_extension.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

typedef Res<T> = Result<T, AppException>;

/// A wrapper around [Result] to make it easier to use with AppException.
FutureOrResult<T> unsafeAsync<T>(FutureOr<T> Function() fn) =>
    Result.tryCatchAsync(
      () async => fn.call(),
      (error) => error.toException(),
    );

abstract class NoParamsAsyncUseCase<T> extends AsyncUseCase<void, T> {
  const NoParamsAsyncUseCase();
}
