// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/core/enums/operation_type.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';

/// {@template crud_data_source}
/// A [BaseDataSource] that provides SCRUD operations.
/// {@endtemplate}
abstract class CrudDataSource extends BaseDataSource {
  /// {@macro crud_data_source}
  const CrudDataSource({
    required this.id,
  });

  final String? Function(
    OperationType operationType,
    Map<String, dynamic> object,
  ) id;

  /// Creates a new object.
  Future<void> create(Map<String, dynamic> object, {String? id});

  /// Gets a object by its [id].
  Future<Map<String, dynamic>?> get(String id);

  /// Gets all objects.
  Future<List<dynamic>?> getAll();

  /// Updates a object by its [id].
  Future<void> update(
    String id, {
    Map<String, dynamic>? object,
  });

  /// Updates all objects.
  Future<void> updateAll(Map<String, Object?>? data);

  /// Deletes a object by its [id].
  Future<void> delete(String id);

  /// Deletes all objects.
  Future<void> deleteAll();

  /// Queries objects by [conditions].
  Future<List<dynamic>?> search(List<Query> conditions);

  /// Streams objects by [conditions].
  Stream<List<dynamic>?> stream({
    String? id,
    List<Query>? conditions,
    bool includeMetadataChanges = false,
  });
}
