// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:wyatt_ui_components/src/core/utils/multi_color.dart';
import 'package:wyatt_ui_components/src/domain/entities/component.dart';

/// {@template card_component}
/// Card component.
/// Used to display a card with a background and a shadow.
/// {@endtemplate}
abstract class CardComponent extends Component {
  const CardComponent({
    this.radius,
    this.padding,
    this.backgroundColors,
    this.borderColors,
    this.stroke,
    this.minSize,
    this.maxSize,
    this.shadow,
    this.titleStyle,
    this.subtitleStyle,
    this.bodyStyle,
    this.background,
    super.key,
  });

  /// Card radius
  ///
  /// Default to `BorderRadius.all(Radius.circular(12.0))`
  final BorderRadiusGeometry? radius;

  /// Padding and gaps of this card
  ///
  /// Default to `Theme.cardTheme.margin`
  final EdgeInsetsGeometry? padding;

  /// Card background gradient colors (from left to right)
  ///
  /// Default to `Theme.cardTheme.color`
  final MultiColor? backgroundColors;

  /// Border colors (from left to right).
  ///
  /// Default to `null`
  final MultiColor? borderColors;

  /// Stroke of the border
  ///
  /// Default to `null`
  final double? stroke;

  /// Drop shadow
  ///
  /// Default to `null`
  final BoxShadow? shadow;

  /// Minimum size of the card
  ///
  /// Default to `const Size(330, double.infinity)`
  final Size? minSize;

  /// Maximum size of the card
  ///
  /// Default to `const Size(double.infinity, double.infinity)`
  final Size? maxSize;

  /// Title text style
  ///
  /// Default to `Theme.textTheme.titleLarge`
  final TextStyle? titleStyle;

  /// Subtitle text style
  ///
  /// Default to `Theme.textTheme.titleMedium`
  final TextStyle? subtitleStyle;

  /// Body text style
  ///
  /// Default to `Theme.textTheme.bodyMedium`
  final TextStyle? bodyStyle;

  /// Background of the card
  final Widget? background;
}
